#include <unistd.h>
#include <cerrno>
#include <cstring>
#include <string>

#include <trampoline/types.h>

#include "../log.h"
#include "main.h"

// Setup Pipes
#define PIPE_READ 0
#define PIPE_WRITE 1
static void safe_pipe(int *out) {
    const int ret = pipe(out);
    if (ret != 0) {
        ERR("Unable To Create Pipe: %s", strerror(errno));
    }
}
void PipeImplementation::init() {
    setenv(MCPI_USE_PIPE_TRAMPOLINE_ENV, "", 1);
    safe_pipe(arguments_pipe);
    safe_pipe(return_value_pipe);
}

// Parent
void PipeImplementation::init_parent() const {
    // Close Unneeded Pipes
    close(arguments_pipe[PIPE_READ]);
    close(return_value_pipe[PIPE_WRITE]);
    // Setup Environment
    setenv(_MCPI_TRAMPOLINE_ARGUMENTS_ENV, std::to_string(arguments_pipe[PIPE_WRITE]).c_str(), true);
    setenv(_MCPI_TRAMPOLINE_RETURN_VALUE_ENV, std::to_string(return_value_pipe[PIPE_READ]).c_str(), true);
}

// Child
void PipeImplementation::init_child() {
    // Close Unneeded Pipes
    close(arguments_pipe[PIPE_WRITE]);
    close(return_value_pipe[PIPE_READ]);
    // Init Trampoline
    trampoline.init(this);
}
void PipeImplementation::loop_child() const {
    // Wait For Commands
    trampoline_pipe_arguments cmd = {};
    while (read(arguments_pipe[PIPE_READ], &cmd, sizeof(trampoline_pipe_arguments)) > 0) {
        static unsigned char args[MAX_TRAMPOLINE_ARGS_SIZE];
        size_t position = 0;
        while (position < cmd.length) {
            const ssize_t ret = read(arguments_pipe[PIPE_READ], args + position, cmd.length - position);
            if (ret == -1) {
                ERR("Unable To Read Trampoline Arguments");
            } else {
                position += ret;
            }
        }
        uint32_t ret = trampoline(cmd.id, args);
        if (!cmd.allow_early_return) {
            const ssize_t ret2 = write(return_value_pipe[PIPE_WRITE], &ret, sizeof(uint32_t));
            if (ret2 == -1) {
                ERR("Unable To Write Trampoline Return Value");
            }
        }
    }
}
