#include <unistd.h>
#include <cstring>
#include <cerrno>
#include <sys/wait.h>
#include <linux/prctl.h>
#include <sys/prctl.h>

#include "../log.h"
#ifdef MCPI_HAS_QEMU
#include "../qemu/qemu.h"
#endif
#include "main.h"

// Main
int PipeImplementation::main(const int argc, char *argv[]) {
    // Setup
    init();
    // Fork
    const pid_t pid = fork();
    if (pid == -1) {
        ERR("Unable To Fork Process: %s", strerror(errno));
    } else if (pid == 0) {
        // Child

        // Setup
        setpgid(0, 0);
        init_child();
        // Read From Pipes
        loop_child();
        // Exit
        return 0;
    } else {
        // Parent/Guest

        // Setup
        init_parent();
        prctl(PR_SET_PTRACER, pid);
        // Execute Program
#ifdef MCPI_HAS_QEMU
        return QEMU::run(argc, argv);
#else
        (void) argc;
        execvp(argv[0], (char *const *) argv);
        ERR("Unable To Execute Program: %s: %s", argv[0], strerror(errno));
#endif
    }
}