#include <sys/uio.h>
#include <cstring>
#include <cerrno>

#include "main.h"
#include "../log.h"
#ifdef MCPI_HAS_QEMU
#include "../qemu/qemu.h"
#endif

// Write Memory To Parent/Guest
void PipeImplementation::memory_writer(const uint32_t guest_addr, const void *data, const uint32_t size) const {
    iovec local[1];
    local[0].iov_base = (void *) data;
    local[0].iov_len = size;
    iovec remote[1];
    void *fixed_guest_addr =
#ifdef MCPI_HAS_QEMU
        QEMU::guest_to_host(guest_addr)
#else
        (void *) (uint64_t) guest_addr
#endif
        ;
    remote[0].iov_base = fixed_guest_addr;
    remote[0].iov_len = size;
    const ssize_t ret = process_vm_writev(guest_pid, local, 1, remote, 1, 0);
    if (ret != size) {
        ERR("Unable To Write Data: 0x%08x: %s", guest_addr, strerror(errno));
    }
}